@@ -2,6 +2,9 @@ from django.core.exceptions import ObjectDoesNotExist
 from django.core.paginator import Paginator, Page
 from django.forms import modelform_factory
 from channels.db import database_sync_to_async
+from django.db.models import F, Q
+from django.db.models.sql.query import FieldError
+from .. import settings as self_settings
 
 __all__ = [
     'AsyncModelBinding',
@@ -36,13 +39,24 @@ class AsyncModelBinding(object):
         else:
             return queryset
 
-    def filter_queryset(self, queryset, data):
+    def filter_queryset(self, queryset, filters, data):
+        for name, data in filters.items():
+            queryset = self.filter_in(queryset, name, data)
         return queryset
 
-    def paginate(self, queryset, data, limit=None):
+    def order_queryset(self, queryset, order, data):
+        if order:
+            queryset = queryset.order_by(
+                F(order.strip('-')).asc(nulls_last=True) if order.startswith('-')
+                else
+                F(order.strip('-')).desc(nulls_last=True)
+            )
+        return queryset
+
+    def paginate(self, queryset, page, limit, data):
         limit = limit or self.page_size or self_settings.DEFAULT_PAGE_SIZE
         try:
-            page = int(data['page'])
+            page = int(page)
         except (KeyError, ValueError, TypeError) as e:
             page = 1
         paginator = Paginator(queryset, max(10, min(100, int(limit))))
@@ -52,11 +66,12 @@ class AsyncModelBinding(object):
         pk = data.get(self.data_pk, None)
         try:
             if isinstance(pk, list):
-                return list(self.filter_queryset(self.model.objects, {}).filter(pk__in=pk))
+                return list(self.get_queryset(data).filter(**{f'{self.data_pk}__in': pk}))
             else:
-                return self.filter_queryset(self.model.objects, {}).get(pk=pk)
+                return self.get_queryset(data).get(**{self.data_pk: pk})
         except self.model.DoesNotExist as e:
             if create:
                 return self.model()
             else:
-                raise self.model.DoesNotExist(f'{self.stream} pk:{pk} Does Not Exist')
+                raise self.model.DoesNotExist(
+                    f'{self.stream} {self.data_pk}:{pk} Does Not Exist')
